/* Norman Cohen, April 1995
 * $Id
 */

#ifndef H_io
#define H_io

void	error(char * fmt, ...);
void	warn(char * fmt, ...);
void	monitor(char * file, int line, char * fmt, ...);
void	reset_func_timer(void);
double	get_func_timer(void);
void	reset_elapse_timer(void);
double	get_elapse_timer(void);
void	phase(char * fmt, ...);
void	record(char * fmt, ...);
void	record_stat(char * fmt, ...);

#if 0
char *	get_grapher_name();
char *	get_temp_file();
void	simple_push_call_stack(char * file, int line);
void	pop_call_stack(void);
void	dump_call_stack(void);


#if(0&&!NDEBUG)
  #define ENTER_CALL()	simple_push_call_stack(__FILE__,__LINE__)
  #define EXIT_CALL()	pop_call_stack()
#else
  #define ENTER_CALL()	((void)0)
  #define EXIT_CALL()	((void)0)
#endif

#endif /* if 0 */

#endif
